@@ -5,28 +5,27 @@
 */
 
 #include "kio_archivebase.h"
+#include <kio_archive_debug.h>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <stdlib.h>
 #include <errno.h>
+#include <stdlib.h>
+#include <sys/stat.h>
 
 #include <KIO/StatJob>
+#include <KTar>
+#include <KZip>
+#include <KAr>
+#include <K7Zip>
+#include <KUser>
+#include <KLocalizedString>
 
 #include <QFile>
 #include <QDir>
 #include <QMimeDatabase>
 #include <QMimeType>
-
 #include <QUrl>
-#include <ktar.h>
-#include <kzip.h>
-#include <kar.h>
-#include <k7zip.h>
-#include <kio/global.h>
-#include <kio_archive_debug.h>
-#include <kuser.h>
-#include <KLocalizedString>
+
+#include <memory>
 
 #ifdef Q_OS_WIN
 #define S_ISDIR(m) (((m & S_IFMT) == S_IFDIR))
@@ -34,7 +33,7 @@
 
 using namespace KIO;
 
-ArchiveProtocolBase::ArchiveProtocolBase( const QByteArray &proto, const QByteArray &pool, const QByteArray &app ) : SlaveBase( proto, pool, app )
+ArchiveProtocolBase::ArchiveProtocolBase( const QByteArray &proto, const QByteArray &pool, const QByteArray &app ) : WorkerBase( proto, pool, app )
 {
     qCDebug(KIO_ARCHIVE_LOG);
     m_archiveFile = nullptr;
@@ -155,7 +154,7 @@ bool ArchiveProtocolBase::checkNewFile( const QUrl & url, QString & path, KIO::E
     // Open new file
     m_archiveFile = this->createArchive( url.scheme(), archiveFile );
     if ( !m_archiveFile ) {
-        qCWarning(KIO_ARCHIVE_LOG) << "Protocol" << url.scheme() << "not supported by this IOSlave" ;
+        qCWarning(KIO_ARCHIVE_LOG) << "Protocol" << url.scheme() << "not supported by this IOWorker" ;
         errorNum = KIO::ERR_UNSUPPORTED_PROTOCOL;
         return false;
     }
@@ -257,7 +256,7 @@ void ArchiveProtocolBase::createUDSEntry( const KArchiveEntry * archiveEntry, UD
     entry.fastInsert( KIO::UDSEntry::UDS_LINK_DEST, archiveEntry->symLinkTarget());
 }
 
-void ArchiveProtocolBase::listDir( const QUrl & url )
+KIO::WorkerResult ArchiveProtocolBase::listDir( const QUrl & url )
 {
     qCDebug(KIO_ARCHIVE_LOG) << url.url();
 
@@ -269,26 +268,23 @@ void ArchiveProtocolBase::listDir( const QUrl & url )
         {
             // If we cannot open, it might be a problem with the archive header (e.g. unsupported format)
             // Therefore give a more specific error message
-            error( KIO::ERR_SLAVE_DEFINED,
+            return KIO::WorkerResult::fail( KIO::ERR_WORKER_DEFINED,
                    i18n( "Could not open the file, probably due to an unsupported file format.\n%1",
                          url.toDisplayString() ) );
-            return;
         }
         else if ( errorNum != ERR_IS_DIRECTORY )
         {
             // We have any other error
-            error( errorNum, url.toDisplayString() );
-            return;
+            return KIO::WorkerResult::fail( errorNum, url.toDisplayString() );
         }
         // It's a real dir -> redirect
         QUrl redir = QUrl::fromLocalFile(url.path());
         qCDebug(KIO_ARCHIVE_LOG) << "Ok, redirection to" << redir.url();
         redirection( redir );
-        finished();
         // And let go of the tar file - for people who want to unmount a cdrom after that
         delete m_archiveFile;
         m_archiveFile = nullptr;
-        return;
+        return KIO::WorkerResult::pass();
     }
 
     if ( path.isEmpty() )
@@ -299,8 +295,7 @@ void ArchiveProtocolBase::listDir( const QUrl & url )
         redir.setPath( url.path() + QLatin1Char('/') );
         qCDebug(KIO_ARCHIVE_LOG) << "redirection" << redir.url();
         redirection( redir );
-        finished();
-        return;
+        return KIO::WorkerResult::pass();
     }
 
     qCDebug(KIO_ARCHIVE_LOG) << "checkNewFile done";
@@ -312,13 +307,11 @@ void ArchiveProtocolBase::listDir( const QUrl & url )
         const KArchiveEntry* e = root->entry( path );
         if ( !e )
         {
-            error( KIO::ERR_DOES_NOT_EXIST, url.toDisplayString() );
-            return;
+            return KIO::WorkerResult::fail( KIO::ERR_DOES_NOT_EXIST, url.toDisplayString() );
         }
         if ( ! e->isDirectory() )
         {
-            error( KIO::ERR_IS_FILE, url.toDisplayString() );
-            return;
+            return KIO::WorkerResult::fail( KIO::ERR_IS_FILE, url.toDisplayString() );
         }
         dir = (KArchiveDirectory*)e;
     } else {
@@ -345,12 +338,11 @@ void ArchiveProtocolBase::listDir( const QUrl & url )
         listEntry(entry);
     }
 
-    finished();
-
     qCDebug(KIO_ARCHIVE_LOG) << "done";
+    return KIO::WorkerResult::pass();
 }
 
-void ArchiveProtocolBase::stat( const QUrl & url )
+KIO::WorkerResult ArchiveProtocolBase::stat( const QUrl & url )
 {
     QString path;
     UDSEntry entry;
@@ -363,16 +355,14 @@ void ArchiveProtocolBase::stat( const QUrl & url )
         {
             // If we cannot open, it might be a problem with the archive header (e.g. unsupported format)
             // Therefore give a more specific error message
-            error( KIO::ERR_SLAVE_DEFINED,
+            return KIO::WorkerResult::fail( KIO::ERR_WORKER_DEFINED,
                    i18n( "Could not open the file, probably due to an unsupported file format.\n%1",
                          url.toDisplayString() ) );
-            return;
         }
         else if ( errorNum != ERR_IS_DIRECTORY )
         {
             // We have any other error
-            error( errorNum, url.toDisplayString() );
-            return;
+            return KIO::WorkerResult::fail( errorNum, url.toDisplayString() );
         }
         entry.reserve(2);
         // Real directory. Return just enough information for KRun to work
@@ -389,20 +379,17 @@ void ArchiveProtocolBase::stat( const QUrl & url )
         if ( QT_STAT( QFile::encodeName(fullPath).constData(), &buff ) == -1 )
         {
             // Should not happen, as the file was already stated by checkNewFile
-            error( KIO::ERR_CANNOT_STAT, url.toDisplayString() );
-            return;
+            return KIO::WorkerResult::fail( KIO::ERR_CANNOT_STAT, url.toDisplayString() );
         }
 
         entry.fastInsert( KIO::UDSEntry::UDS_FILE_TYPE, buff.st_mode & S_IFMT);
 
         statEntry( entry );
 
-        finished();
-
         // And let go of the tar file - for people who want to unmount a cdrom after that
         delete m_archiveFile;
         m_archiveFile = nullptr;
-        return;
+        return KIO::WorkerResult::pass();
     }
 
     const KArchiveDirectory* root = m_archiveFile->directory();
@@ -416,8 +403,7 @@ void ArchiveProtocolBase::stat( const QUrl & url )
     }
     if ( !archiveEntry )
     {
-        error( KIO::ERR_DOES_NOT_EXIST, url.toDisplayString() );
-        return;
+        return KIO::WorkerResult::fail( KIO::ERR_DOES_NOT_EXIST, url.toDisplayString() );
     }
 
     if (archiveEntry == root) {
@@ -435,10 +421,10 @@ void ArchiveProtocolBase::stat( const QUrl & url )
     }
     statEntry( entry );
 
-    finished();
+    return KIO::WorkerResult::pass();
 }
 
-void ArchiveProtocolBase::get( const QUrl & url )
+KIO::WorkerResult ArchiveProtocolBase::get( const QUrl & url )
 {
     qCDebug(KIO_ARCHIVE_LOG) << url.url();
 
@@ -450,16 +436,14 @@ void ArchiveProtocolBase::get( const QUrl & url )
         {
             // If we cannot open, it might be a problem with the archive header (e.g. unsupported format)
             // Therefore give a more specific error message
-            error( KIO::ERR_SLAVE_DEFINED,
+            return KIO::WorkerResult::fail( KIO::ERR_WORKER_DEFINED,
                    i18n( "Could not open the file, probably due to an unsupported file format.\n%1",
                          url.toDisplayString() ) );
-            return;
         }
         else
         {
             // We have any other error
-            error( errorNum, url.toDisplayString() );
-            return;
+            return KIO::WorkerResult::fail( errorNum, url.toDisplayString() );
         }
     }
 
@@ -468,13 +452,11 @@ void ArchiveProtocolBase::get( const QUrl & url )
 
     if ( !archiveEntry )
     {
-        error( KIO::ERR_DOES_NOT_EXIST, url.toDisplayString() );
-        return;
+        return KIO::WorkerResult::fail( KIO::ERR_DOES_NOT_EXIST, url.toDisplayString() );
     }
     if ( archiveEntry->isDirectory() )
     {
-        error( KIO::ERR_IS_DIRECTORY, url.toDisplayString() );
-        return;
+        return KIO::WorkerResult::fail( KIO::ERR_IS_DIRECTORY, url.toDisplayString() );
     }
     const KArchiveFile* archiveFileEntry = static_cast<const KArchiveFile *>(archiveEntry);
     if ( !archiveEntry->symLinkTarget().isEmpty() )
@@ -484,8 +466,7 @@ void ArchiveProtocolBase::get( const QUrl & url )
         const QUrl realURL = url.resolved(QUrl(target));
         qCDebug(KIO_ARCHIVE_LOG) << "realURL=" << realURL;
         redirection( realURL );
-        finished();
-        return;
+        return KIO::WorkerResult::pass();
     }
 
     //qCDebug(KIO_ARCHIVE_LOG) << "Preparing to get the archive data";
@@ -497,21 +478,18 @@ void ArchiveProtocolBase::get( const QUrl & url )
      * - errors are skipped, resulting in an empty file
      */
 
-    QIODevice* io = archiveFileEntry->createDevice();
+    std::unique_ptr<QIODevice> io(archiveFileEntry->createDevice());
 
     if (!io)
     {
-        error( KIO::ERR_SLAVE_DEFINED,
+        return KIO::WorkerResult::fail( KIO::ERR_WORKER_DEFINED,
                i18n( "The archive file could not be opened, perhaps because the format is unsupported.\n%1",
                      url.toDisplayString() ) );
-        return;
     }
 
     if ( !io->open( QIODevice::ReadOnly ) )
     {
-        error( KIO::ERR_CANNOT_OPEN_FOR_READING, url.toDisplayString() );
-        delete io;
-        return;
+        return KIO::WorkerResult::fail( KIO::ERR_CANNOT_OPEN_FOR_READING, url.toDisplayString() );
     }
 
     totalSize( archiveFileEntry->size() );
@@ -525,9 +503,7 @@ void ArchiveProtocolBase::get( const QUrl & url )
     if ( buffer.isEmpty() && bufferSize > 0 )
     {
         // Something went wrong
-        error( KIO::ERR_OUT_OF_MEMORY, url.toDisplayString() );
-        delete io;
-        return;
+        return KIO::WorkerResult::fail( KIO::ERR_OUT_OF_MEMORY, url.toDisplayString() );
     }
 
     bool firstRead = true;
@@ -547,9 +523,7 @@ void ArchiveProtocolBase::get( const QUrl & url )
         if ( read != bufferSize )
         {
             qCWarning(KIO_ARCHIVE_LOG) << "Read" << read << "bytes but expected" << bufferSize ;
-            error( KIO::ERR_CANNOT_READ, url.toDisplayString() );
-            delete io;
-            return;
+            return KIO::WorkerResult::fail( KIO::ERR_CANNOT_READ, url.toDisplayString() );
         }
         if ( firstRead )
         {
@@ -567,11 +541,10 @@ void ArchiveProtocolBase::get( const QUrl & url )
         fileSize -= bufferSize;
     }
     io->close();
-    delete io;
 
     data( QByteArray() );
 
-    finished();
+    return KIO::WorkerResult::pass();
 }
 
 /*
