@@ -10,6 +10,7 @@
 #include <QCoreApplication>
 #include <QUrl>
 #include <QUrlQuery>
+#include <QDataStream>
 
 #include <KIO/Job>
 #include <KLocalizedString>
@@ -31,21 +32,21 @@ using namespace KAStats::Terms;
 class KIOPluginForMetaData : public QObject
 {
     Q_OBJECT
-    Q_PLUGIN_METADATA(IID "org.kde.kio.slave.recentlyused" FILE "recentlyused.json")
+    Q_PLUGIN_METADATA(IID "org.kde.kio.worker.recentlyused" FILE "recentlyused.json")
 };
 
 extern "C" int Q_DECL_EXPORT kdemain(int argc, char **argv)
 {
-    // necessary to use other kio slaves
+    // necessary to use other kio workers
     QCoreApplication app(argc, argv);
     app.setApplicationName(QStringLiteral("kio_recentlyused"));
     if (argc != 4) {
         fprintf(stderr, "Usage: kio_recentlyused protocol domain-socket1 domain-socket2\n");
         exit(-1);
     }
-    // start the slave
-    RecentlyUsed slave(argv[2], argv[3]);
-    slave.dispatchLoop();
+    // start the worker
+    RecentlyUsed worker(argv[2], argv[3]);
+    worker.dispatchLoop();
     return 0;
 }
 
@@ -56,7 +57,7 @@ static bool isRootUrl(const QUrl &url)
 }
 
 RecentlyUsed::RecentlyUsed(const QByteArray &pool, const QByteArray &app)
-    : SlaveBase("recentlyused", pool, app)
+    : WorkerBase("recentlyused", pool, app)
 {
 }
 
@@ -162,7 +163,7 @@ ResultModel *runQuery(const QUrl &url)
     return new ResultModel(query);
 }
 
-KIO::UDSEntry RecentlyUsed::udsEntryFromResource(const QString &resource)
+KIO::UDSEntry RecentlyUsed::udsEntryFromResource(const QString &resource, const QString &mimeType, int row)
 {
     qCDebug(KIO_RECENTLYUSED_LOG) << "udsEntryFromResource" << resource;
 
@@ -174,38 +175,46 @@ KIO::UDSEntry RecentlyUsed::udsEntryFromResource(const QString &resource)
 
     // we do not want to wait for the event loop to delete the job
     QScopedPointer<KIO::StatJob> sp(job);
-    job->setAutoDelete(false);
-    if (job->exec()) {
-        uds = job->statResult();
+    sp->setAutoDelete(false);
+    if (sp->exec()) {
+        uds = sp->statResult();
     }
-    uds.fastInsert(KIO::UDSEntry::UDS_URL, resourceUrl.toString());
+    // replace name with a technical unique name
+    const auto name = uds.stringValue(KIO::UDSEntry::UDS_NAME);
+    uds.replace(KIO::UDSEntry::UDS_NAME, QStringLiteral("%1-%2").arg(name).arg(row));
+    uds.reserve(uds.count() + 4);
+    uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_NAME, name);
+    uds.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, mimeType);
+    uds.fastInsert(KIO::UDSEntry::UDS_TARGET_URL, resourceUrl.toString());
+    uds.fastInsert(KIO::UDSEntry::UDS_LOCAL_PATH, resource);
     return uds;
 }
 
-void RecentlyUsed::listDir(const QUrl &url)
+KIO::WorkerResult RecentlyUsed::listDir(const QUrl &url)
 {
     if (!isRootUrl(url)) {
         const auto path = url.path();
         if (path != QStringLiteral("/files") && path != QStringLiteral("/locations") ) {
-            error(KIO::ERR_DOES_NOT_EXIST, url.toDisplayString());
-            return;
+            return KIO::WorkerResult::fail(KIO::ERR_DOES_NOT_EXIST, url.toDisplayString());
         }
     }
 
-    auto model = runQuery(url);
+    const auto model = runQuery(url);
 
     KIO::UDSEntryList udslist;
     udslist.reserve(model->rowCount());
 
-    for (int r = 0; r < model->rowCount(); ++r) {
-        QModelIndex index = model->index(r, 0);
-        QString resource = model->data(index, ResultModel::ResourceRole).toString();
+    for (int row = 0; row < model->rowCount(); ++row) {
+        const QModelIndex index = model->index(row, 0);
+        const QString resource = model->data(index, ResultModel::ResourceRole).toString();
+        const QString mimeType = model->data(index, ResultModel::MimeType).toString();
 
-        udslist << udsEntryFromResource(resource);
+        udslist << udsEntryFromResource(resource, mimeType, row);
     }
 
     listEntries(udslist);
-    finished();
+
+    return KIO::WorkerResult::pass();
 }
 
 KIO::UDSEntry RecentlyUsed::udsEntryForRoot(const QString &dirName, const QString &iconName)
@@ -226,7 +235,7 @@ KIO::UDSEntry RecentlyUsed::udsEntryForRoot(const QString &dirName, const QStrin
     return uds;
 }
 
-void RecentlyUsed::stat(const QUrl &url)
+KIO::WorkerResult RecentlyUsed::stat(const QUrl &url)
 {
     qCDebug(KIO_RECENTLYUSED_LOG) << "stating" << " " << url;
 
@@ -238,35 +247,71 @@ void RecentlyUsed::stat(const QUrl &url)
         const QString dirName = i18n("Recent Documents");
 
         statEntry(udsEntryForRoot(dirName, QStringLiteral("document-open-recent")));
-        finished();
-    } else {
-
-        const auto path = url.path();
-        if (path == QStringLiteral("/files")) {
-            const QString dirName = i18n("Recent Files");
-            statEntry(udsEntryForRoot(dirName, QStringLiteral("document-open-recent")));
-            finished();
-        } else if (path == QStringLiteral("/locations")) {
-            const QString dirName = i18n("Recent Locations");
-            statEntry(udsEntryForRoot(dirName, QStringLiteral("folder-open-recent")));
-            finished();
-        } else {
-            // only / /files and /locations paths are supported
-            error(KIO::ERR_DOES_NOT_EXIST, url.toDisplayString());
-        }
+        return KIO::WorkerResult::pass();
     }
+
+    const auto path = url.path();
+    if (path == QStringLiteral("/files")) {
+        const QString dirName = i18n("Recent Files");
+        statEntry(udsEntryForRoot(dirName, QStringLiteral("document-open-recent")));
+    } else if (path == QStringLiteral("/locations")) {
+        const QString dirName = i18n("Recent Locations");
+        statEntry(udsEntryForRoot(dirName, QStringLiteral("folder-open-recent")));
+    } else {
+        // only / /files and /locations paths are supported
+        return KIO::WorkerResult::fail(KIO::ERR_DOES_NOT_EXIST, url.toDisplayString());
+    }
+    return KIO::WorkerResult::pass();
 }
 
-void RecentlyUsed::mimetype(const QUrl &url)
+KIO::WorkerResult RecentlyUsed::mimetype(const QUrl &url)
 {
     // the root url is always a folder
     if (isRootUrl(url)) {
         mimeType(QStringLiteral("inode/directory"));
-        finished();
-    } else {
-        // only the root path is supported
-        error(KIO::ERR_DOES_NOT_EXIST, url.toDisplayString());
+        return KIO::WorkerResult::pass();
     }
+
+    // only the root path is supported
+    return KIO::WorkerResult::fail(KIO::ERR_DOES_NOT_EXIST, url.toDisplayString());
+}
+
+KIO::WorkerResult RecentlyUsed::special(const QByteArray &data) {
+
+    int id;
+    QDataStream stream(data);
+    stream >> id;
+
+    switch (id) {
+    case 1: { // Forget
+        QList<QUrl> urls;
+        stream >> urls;
+
+        QList<QString> paths;
+        for (const auto &url: qAsConst(urls)) {
+            if (url.isLocalFile() || url.scheme().isEmpty()) {
+                paths.append(url.path());
+            } else {
+                paths.append(url.toString());
+            }
+        }
+
+        Query query = UsedResources
+                | Limit(paths.size());
+        query.setUrlFilters(Url(paths));
+        query.setAgents(Agent::any());
+        query.setActivities(Activity::any());
+
+        ResultModel model(query);
+        model.forgetResources(paths);
+
+        break;
+    }
+    default:
+        break;
+    }
+
+    return KIO::WorkerResult::pass();
 }
 
 // needed for JSON file embedding
