@@ -10,14 +10,16 @@
 
 #include <QMimeDatabase>
 #include <QMimeType>
+#include <QScopeGuard>
 #include <QVarLengthArray>
+
 #include <KLocalizedString>
 
 #include <future>
 
 #include "transfer.h"
 
-void SMBSlave::get(const QUrl &kurl)
+WorkerResult SMBWorker::get(const QUrl &kurl)
 {
     qCDebug(KIO_SMB_LOG) << kurl;
 
@@ -26,28 +28,24 @@ void SMBSlave::get(const QUrl &kurl)
     // if URL is not valid we have to redirect to correct URL
     if (kvurl != kurl) {
         redirection(kvurl);
-        finished();
-        return;
+        return WorkerResult::pass();
     }
 
     if (!m_context.isValid()) {
-        SlaveBase::error(ERR_INTERNAL, i18n("libsmbclient failed to create context"));
-        return;
+        return WorkerResult::fail(ERR_INTERNAL, i18n("libsmbclient failed to create context"));
     }
 
     // Stat
     SMBUrl url = kurl;
     int errNum = cache_stat(url, &st);
     if (errNum != 0) {
-        if (errNum == EACCES)
-            error(KIO::ERR_ACCESS_DENIED, url.toDisplayString());
-        else
-            error(KIO::ERR_DOES_NOT_EXIST, url.toDisplayString());
-        return;
+        if (errNum == EACCES) {
+            return WorkerResult::fail(KIO::ERR_ACCESS_DENIED, url.toDisplayString());
+        }
+        return WorkerResult::fail(KIO::ERR_DOES_NOT_EXIST, url.toDisplayString());
     }
     if (S_ISDIR(st.st_mode)) {
-        error(KIO::ERR_IS_DIRECTORY, url.toDisplayString());
-        return;
+        return WorkerResult::fail(KIO::ERR_IS_DIRECTORY, url.toDisplayString());
     }
 
     // Set the total size
@@ -56,9 +54,11 @@ void SMBSlave::get(const QUrl &kurl)
     // Open and read the file
     int filefd = smbc_open(url.toSmbcUrl(), O_RDONLY, 0);
     if (filefd < 0) {
-        error(KIO::ERR_CANNOT_OPEN_FOR_READING, url.toDisplayString());
-        return;
+        return WorkerResult::fail(KIO::ERR_CANNOT_OPEN_FOR_READING, url.toDisplayString());
     }
+    auto filefdClose = qScopeGuard([filefd] {
+        smbc_close(filefd);
+    });
 
     KIO::filesize_t totalbytesread = 0;
     QByteArray filedata;
@@ -105,20 +105,19 @@ void SMBSlave::get(const QUrl &kurl)
         buffer.unpop();
     }
     if (future.get() != KJob::NoError) { // check if read had an error
-        error(future.get(), url.toDisplayString());
+        return WorkerResult::fail(future.get(), url.toDisplayString());
     }
 
-    smbc_close(filefd);
     data(QByteArray());
     if (totalbytesread != static_cast<KIO::filesize_t>(st.st_size)) {
         qCWarning(KIO_SMB_LOG) << "Got" << totalbytesread << "bytes but expected" << st.st_size;
     }
     processedSize(static_cast<KIO::filesize_t>(st.st_size));
 
-    finished();
+    return WorkerResult::pass();
 }
 
-void SMBSlave::open(const QUrl &kurl, QIODevice::OpenMode mode)
+WorkerResult SMBWorker::open(const QUrl &kurl, QIODevice::OpenMode mode)
 {
     int errNum = 0;
     qCDebug(KIO_SMB_LOG) << kurl;
@@ -129,33 +128,29 @@ void SMBSlave::open(const QUrl &kurl, QIODevice::OpenMode mode)
     // if URL is not valid we have to redirect to correct URL
     if (kvurl != kurl) {
         redirection(kvurl);
-        finished();
-        return;
+        return WorkerResult::pass();
     }
 
     if (!m_context.isValid()) {
-        error(KIO::ERR_ACCESS_DENIED, kurl.toDisplayString());
-        return;
+        return WorkerResult::fail(KIO::ERR_ACCESS_DENIED, kurl.toDisplayString());
     }
 
     // Save the URL as a private member
     // FIXME For some reason m_openUrl has be be declared in bottom private
-    // section of the class SMBSlave declaration instead of the top section
+    // section of the class SMBWorker declaration instead of the top section
     // or else this assignment fails
     m_openUrl = kurl;
 
     // Stat
     errNum = cache_stat(m_openUrl, &st);
     if (errNum != 0) {
-        if (errNum == EACCES)
-            error(KIO::ERR_ACCESS_DENIED, m_openUrl.toDisplayString());
-        else
-            error(KIO::ERR_DOES_NOT_EXIST, m_openUrl.toDisplayString());
-        return;
+        if (errNum == EACCES) {
+            return WorkerResult::fail(KIO::ERR_ACCESS_DENIED, m_openUrl.toDisplayString());
+        }
+        return WorkerResult::fail(KIO::ERR_DOES_NOT_EXIST, m_openUrl.toDisplayString());
     }
     if (S_ISDIR(st.st_mode)) {
-        error(KIO::ERR_IS_DIRECTORY, m_openUrl.toDisplayString());
-        return;
+        return WorkerResult::fail(KIO::ERR_IS_DIRECTORY, m_openUrl.toDisplayString());
     }
 
     // Set the total size
@@ -182,12 +177,11 @@ void SMBSlave::open(const QUrl &kurl, QIODevice::OpenMode mode)
     // Open the file
     m_openFd = smbc_open(m_openUrl.toSmbcUrl(), flags, 0);
     if (m_openFd < 0) {
-        error(KIO::ERR_CANNOT_OPEN_FOR_READING, m_openUrl.toDisplayString());
-        return;
+        return WorkerResult::fail(KIO::ERR_CANNOT_OPEN_FOR_READING, m_openUrl.toDisplayString());
     }
 
     // Determine the mimetype of the file to be retrieved, and emit it.
-    // This is mandatory in all slaves (for KRun/BrowserRun to work).
+    // This is mandatory in all workers (for KRun/BrowserRun to work).
     // If we're not opening the file ReadOnly or ReadWrite, don't attempt to
     // read the file and send the mimetype.
     if (mode & QIODevice::ReadOnly) {
@@ -196,29 +190,26 @@ void SMBSlave::open(const QUrl &kurl, QIODevice::OpenMode mode)
         QVarLengthArray<char> buffer(bytesRequested);
         bytesRead = smbc_read(m_openFd, buffer.data(), bytesRequested);
         if (bytesRead < 0) {
-            error(KIO::ERR_CANNOT_READ, m_openUrl.toDisplayString());
             closeWithoutFinish();
-            return;
-        } else {
-            QByteArray fileData = QByteArray::fromRawData(buffer.data(), bytesRead);
-            QMimeDatabase db;
-            QMimeType type = db.mimeTypeForFileNameAndData(m_openUrl.fileName(), fileData);
-            mimeType(type.name());
+            return WorkerResult::fail(KIO::ERR_CANNOT_READ, m_openUrl.toDisplayString());
+        }
+        QByteArray fileData = QByteArray::fromRawData(buffer.data(), bytesRead);
+        QMimeDatabase db;
+        QMimeType type = db.mimeTypeForFileNameAndData(m_openUrl.fileName(), fileData);
+        mimeType(type.name());
 
-            off_t res = smbc_lseek(m_openFd, 0, SEEK_SET);
-            if (res == (off_t)-1) {
-                error(KIO::ERR_CANNOT_SEEK, m_openUrl.path());
-                closeWithoutFinish();
-                return;
-            }
+        off_t res = smbc_lseek(m_openFd, 0, SEEK_SET);
+        if (res == (off_t)-1) {
+            closeWithoutFinish();
+            return WorkerResult::fail(KIO::ERR_CANNOT_SEEK, m_openUrl.path());
         }
     }
 
     position(0);
-    opened();
+    return WorkerResult::pass();
 }
 
-void SMBSlave::read(KIO::filesize_t bytesRequested)
+WorkerResult SMBWorker::read(KIO::filesize_t bytesRequested)
 {
     Q_ASSERT(m_openFd != -1);
 
@@ -230,16 +221,16 @@ void SMBSlave::read(KIO::filesize_t bytesRequested)
 
     if (bytesRead < 0) {
         qCDebug(KIO_SMB_LOG) << "Could not read " << m_openUrl;
-        error(KIO::ERR_CANNOT_READ, m_openUrl.toDisplayString());
         closeWithoutFinish();
-        return;
+        return WorkerResult::fail(KIO::ERR_CANNOT_READ, m_openUrl.toDisplayString());
     }
 
     QByteArray fileData = QByteArray::fromRawData(buffer.data(), bytesRead);
     data(fileData);
+    return WorkerResult::pass();
 }
 
-void SMBSlave::write(const QByteArray &fileData)
+WorkerResult SMBWorker::write(const QByteArray &fileData)
 {
     Q_ASSERT(m_openFd != -1);
 
@@ -248,50 +239,50 @@ void SMBSlave::write(const QByteArray &fileData)
     ssize_t size = smbc_write(m_openFd, buf.data(), buf.size());
     if (size < 0) {
         qCDebug(KIO_SMB_LOG) << "Could not write to " << m_openUrl;
-        error(KIO::ERR_CANNOT_WRITE, m_openUrl.toDisplayString());
         closeWithoutFinish();
-        return;
+        return WorkerResult::fail(KIO::ERR_CANNOT_WRITE, m_openUrl.toDisplayString());
     }
 
     written(size);
+    return WorkerResult::pass();
 }
 
-void SMBSlave::seek(KIO::filesize_t offset)
+WorkerResult SMBWorker::seek(KIO::filesize_t offset)
 {
     off_t res = smbc_lseek(m_openFd, static_cast<off_t>(offset), SEEK_SET);
     if (res == (off_t)-1) {
-        error(KIO::ERR_CANNOT_SEEK, m_openUrl.path());
         closeWithoutFinish();
-    } else {
-        qCDebug(KIO_SMB_LOG) << "res" << res;
-        position(res);
+        return WorkerResult::fail(KIO::ERR_CANNOT_SEEK, m_openUrl.path());
     }
+    qCDebug(KIO_SMB_LOG) << "res" << res;
+    position(res);
+    return WorkerResult::pass();
 }
 
-void SMBSlave::truncate(KIO::filesize_t length)
+WorkerResult SMBWorker::truncate(KIO::filesize_t length)
 {
     off_t res = smbc_ftruncate(m_openFd, static_cast<off_t>(length));
     if (res < 0) {
-        error(KIO::ERR_CANNOT_TRUNCATE, m_openUrl.path());
         closeWithoutFinish();
-    } else {
-        qCDebug(KIO_SMB_LOG) << "res" << res;
-        truncated(length);
+        return WorkerResult::fail(KIO::ERR_CANNOT_TRUNCATE, m_openUrl.path());
     }
+    qCDebug(KIO_SMB_LOG) << "res" << res;
+    truncated(length);
+    return WorkerResult::pass();
 }
 
-void SMBSlave::closeWithoutFinish()
+void SMBWorker::closeWithoutFinish()
 {
     smbc_close(m_openFd);
 }
 
-void SMBSlave::close()
+WorkerResult SMBWorker::close()
 {
     closeWithoutFinish();
-    finished();
+    return WorkerResult::pass();
 }
 
-void SMBSlave::put(const QUrl &kurl, int permissions, KIO::JobFlags flags)
+WorkerResult SMBWorker::put(const QUrl &kurl, int permissions, KIO::JobFlags flags)
 {
     void *buf;
     size_t bufsize;
@@ -312,12 +303,10 @@ void SMBSlave::put(const QUrl &kurl, int permissions, KIO::JobFlags flags)
     if (exists && !(flags & KIO::Overwrite) && !(flags & KIO::Resume)) {
         if (S_ISDIR(st.st_mode)) {
             qCDebug(KIO_SMB_LOG) << kurl << " already isdir !!";
-            error(KIO::ERR_DIR_ALREADY_EXIST, m_current_url.toDisplayString());
-        } else {
-            qCDebug(KIO_SMB_LOG) << kurl << " already exist !!";
-            error(KIO::ERR_FILE_ALREADY_EXIST, m_current_url.toDisplayString());
+            return WorkerResult::fail(KIO::ERR_DIR_ALREADY_EXIST, m_current_url.toDisplayString());
         }
-        return;
+        qCDebug(KIO_SMB_LOG) << kurl << " already exist !!";
+        return WorkerResult::fail(KIO::ERR_FILE_ALREADY_EXIST, m_current_url.toDisplayString());
     }
 
     if (exists && !(flags & KIO::Resume) && (flags & KIO::Overwrite)) {
@@ -345,7 +334,7 @@ void SMBSlave::put(const QUrl &kurl, int permissions, KIO::JobFlags flags)
         if (permissions != -1) {
             mode = permissions | S_IWUSR | S_IRUSR;
         } else {
-            mode = 600; // 0666;
+            mode = S_IWUSR | S_IRUSR;
         }
 
         qCDebug(KIO_SMB_LOG) << "NO resume " << m_current_url.toSmbcUrl();
@@ -360,13 +349,14 @@ void SMBSlave::put(const QUrl &kurl, int permissions, KIO::JobFlags flags)
     if (filefd < 0) {
         if (errNum == EACCES) {
             qCDebug(KIO_SMB_LOG) << "error " << kurl << " access denied !!";
-            error(KIO::ERR_WRITE_ACCESS_DENIED, m_current_url.toDisplayString());
-        } else {
-            qCDebug(KIO_SMB_LOG) << "error " << kurl << " can not open for writing !!";
-            error(KIO::ERR_CANNOT_OPEN_FOR_WRITING, m_current_url.toDisplayString());
+            return WorkerResult::fail(KIO::ERR_WRITE_ACCESS_DENIED, m_current_url.toDisplayString());
         }
-        return;
+        qCDebug(KIO_SMB_LOG) << "error " << kurl << " can not open for writing !!";
+        return WorkerResult::fail(KIO::ERR_CANNOT_OPEN_FOR_WRITING, m_current_url.toDisplayString());
     }
+    auto closeFileFd = qScopeGuard([filefd] {
+        smbc_close(filefd);
+    });
 
     // Loop until we got 0 (end of data)
     while (true) {
@@ -384,8 +374,7 @@ void SMBSlave::put(const QUrl &kurl, int permissions, KIO::JobFlags flags)
         ssize_t size = smbc_write(filefd, buf, bufsize);
         if (size < 0) {
             qCDebug(KIO_SMB_LOG) << "error " << kurl << "could not write !!";
-            error(KIO::ERR_CANNOT_WRITE, m_current_url.toDisplayString());
-            return;
+            return WorkerResult::fail(KIO::ERR_CANNOT_WRITE, m_current_url.toDisplayString());
         }
         qCDebug(KIO_SMB_LOG) << "wrote " << size;
     }
@@ -393,8 +382,7 @@ void SMBSlave::put(const QUrl &kurl, int permissions, KIO::JobFlags flags)
 
     if (smbc_close(filefd) < 0) {
         qCDebug(KIO_SMB_LOG) << kurl << "could not write !!";
-        error(KIO::ERR_CANNOT_WRITE, m_current_url.toDisplayString());
-        return;
+        return WorkerResult::fail(KIO::ERR_CANNOT_WRITE, m_current_url.toDisplayString());
     }
 
     // set final permissions, if the file was just created
@@ -406,6 +394,5 @@ void SMBSlave::put(const QUrl &kurl, int permissions, KIO::JobFlags flags)
 
     applyMTimeSMBC(m_current_url);
 
-    // We have done our job => finish
-    finished();
+    return WorkerResult::pass();
 }
